unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, StdActns,
  ActnList, ToolWin, ImgList;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileCloseItem: TMenuItem;
    Window1: TMenuItem;
    Help1: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    Edit1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    StatusBar: TStatusBar;
    ActionList1: TActionList;
    EditCut1: TEditCut;
    EditCopy1: TEditCopy;
    EditPaste1: TEditPaste;
    FileNew1: TAction;
    FileSave1: TAction;
    FileExit1: TAction;
    FileOpen1: TAction;
    FileSaveAs1: TAction;
    WindowCascade1: TWindowCascade;
    WindowTileHorizontal1: TWindowTileHorizontal;
    WindowArrangeAll1: TWindowArrange;
    WindowMinimizeAll1: TWindowMinimizeAll;
    HelpAbout1: TAction;
    FileClose1: TWindowClose;
    WindowTileVertical1: TWindowTileVertical;
    WindowTileItem2: TMenuItem;
    ToolBar2: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton9: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ImageList1: TImageList;
    Label1: TLabel;
    Bevel1: TBevel;
    Label2: TLabel;
    procedure FileNew1Execute(Sender: TObject);
    procedure FileOpen1Execute(Sender: TObject);
    procedure HelpAbout1Execute(Sender: TObject);
    procedure FileExit1Execute(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Image1Click(Sender: TObject);
  private
    { Private declarations }

    {----------------- for MDI only --------------------------}
    NewFormProc: TFarProc;
    OldFormProc: TFarProc;
    procedure HookWndProc(var Message: TMessage);
    {----------------- for SDI or MDI ------------------------}
    procedure WMMove(var Message: TWMMove); message WM_MOVE;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    {---------------------------------------------------------}

    procedure CreateMDIChild(const Name: string);

  public
    { Public declarations }

    {----------------- for MDI only --------------------------}
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    {---------------------------------------------------------}

  end;

procedure PaintWithDesktop(AForm: TForm);


var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses ChildWin, About;

{-------------------- for SDI or MDI ---------------------------}
procedure PaintClientWithDesktop(AForm: TForm);
{ handling MDI Client Window }
var
  h: HWND;
begin
  with AForm do begin
    if FormStyle <> fsMDIForm then
      Exit;  { or raise an Exception }
    { paint Client Window with Desktop }
    h:=getDC(ClientHandle);
    if h<>0 then begin
      PaintDesktop(h);
      ReleaseDC(ClientHandle,h);
    end;
  end; { with }
end;

procedure PaintWithDesktop(AForm: TForm);
{ last try : repainting Graphic controls and handling MDI }
var
  i: integer;
begin
  with AForm do begin
    { simply paint Desktop on Canvas }
    PaintDeskTop(Canvas.handle);
    if FormStyle = fsMDIForm then begin
      { need to deal also with Client Window to paint Desktop }
      PaintClientWithDesktop(AForm);
      { repaint also the MDI Children Forms }
      for i:= MDIChildCount-1 downto 0 do
        PaintWithDesktop(MDIChildren[i]);
    end;
    { ask for a repaint of all Graphic controls}
    for i:=0 to ControlCount-1 do
      if Controls[i] is TGraphicControl then begin
        { ******    SPECIAL TRICK : USE WITH CARE    ******     }
        { As Canvas is a protected property in TGraphicControl, }
        { we'll use the simplest descendant to surface it       }
        Controls[i].perform(WM_PAINT, TLabel(Controls[i]).canvas.handle, 0);
      end;
  end; { with }
end;

procedure TMainForm.WMMove(var Message: TWMMove);
begin
  inherited;
  { wait to prevent repainting the screen for MDI applications }
  if not (csLoading in componentState) then
    PaintWithDesktop(self);
end;

procedure TMainForm.WMPaint(var Message: TWMPaint);
begin
  inherited;
  PaintWithDesktop(self);
end;

{-------------------- for MDI only -----------------------------}

procedure TMainForm.CreateWnd;
begin
  inherited CreateWnd;

  if FormStyle <> fsMDIForm then exit;

  { maximum safety ! }
  OldFormProc := nil;
  NewFormProc := nil;
  { get old Client WindowProc }
  OldFormProc := TFarProc(GetWindowLong(ClientHandle, GWL_WNDPROC));
  { install the new proc }
  NewFormProc := MakeObjectInstance(HookWndProc);
  SetWindowLong(ClientHandle, GWL_WNDPROC, LongInt(NewFormProc));
end;

procedure TMainForm.DestroyWnd;
begin
  if Assigned(OldFormProc) then begin
    { install the old proc back }
    SetWindowLong(ClientHandle, GWL_WNDPROC, LongInt(OldFormProc));
    OldFormProc := nil;

    if Assigned(NewFormProc) then
      { some cleaning }
      FreeObjectInstance(NewFormProc);
    NewFormProc := nil;
  end;

  inherited Destroy;
end;

procedure TMainForm.HookWndProc(var Message: TMessage);
  procedure DefaultProc;
  begin
    with Message do
      Result := CallWindowProc(OldFormProc, ClientHandle, Msg, wParam, lParam);
  end;
begin
  with Message do
    case Msg of
      WM_PAINT:
        begin
          DefaultProc;
          PaintClientWithDesktop(Self); { <=== that's the added value }
        end;
    else
      DefaultProc;
    end;
end;  { HookWndProc }

{-------------------- That's all Folks ! -----------------------}



procedure TMainForm.CreateMDIChild(const Name: string);
var
  Child: TMDIChild;
begin
  { create a new MDI child window }
  Child := TMDIChild.Create(Application);
  Child.Caption := Name;
  if FileExists(Name) then Child.Memo1.Lines.LoadFromFile(Name);
end;

procedure TMainForm.FileNew1Execute(Sender: TObject);
begin
  CreateMDIChild('NONAME' + IntToStr(MDIChildCount + 1));
end;

procedure TMainForm.FileOpen1Execute(Sender: TObject);
begin
  if OpenDialog.Execute then
    CreateMDIChild(OpenDialog.FileName);
end;

procedure TMainForm.HelpAbout1Execute(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainForm.FileExit1Execute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  PaintWithDesktop(self);
end;


procedure TMainForm.Image1Click(Sender: TObject);
begin
ShowMessage('rrr');
end;

end.
